from authy import fetch_json as authy_fetch_json, object_to_cotp_json as authy_converter
from gauth import get_accounts as gauth_converter
from mauth import get_accounts as mauth_converter
from freeotp import fetch_json as freeotp_fetch_json, object_to_cotp_json as freeotp_converter


def test_gauth_converter():
    assert gauth_converter("example_databases/gauth.sqlite3") == [
        {"label": "SHA512 10", "secret": "V24KSXF7TXJFGCKG35ZHQVQB4XZK26XPD7UCXU5GXU3POBTXAFLD47SA", "issuer": None,
         "digits": 6, "type": "TOTP", "counter": 0, "algorithm": "SHA1"},
        {"label": "SHA512", "secret": "V24KSXF7TXJFGCKG35ZHQVQB4XZK26XPD7UCXU5GXU3POBTXAFLD47SA", "issuer": None,
         "digits": 6, "type": "TOTP", "counter": 0, "algorithm": "SHA1"}]


def test_authy_converter():
    assert authy_converter(authy_fetch_json("example_databases/authy.xml")) == [
        {"label": "Test", "secret": "TLQEUBNC4ENYRDMLM2ZMQPN7PE272AW7", "issuer": "", "digits": 6, "type": "TOTP", "counter": 0, "algorithm": "SHA1"}]


def test_mauth_converter():
    assert mauth_converter("example_databases/microsoft_authenticator/PhoneFactor") == [
        {"label": "Test1", "secret": "XHRHVLZKO5YARFKEH65RFC7NVOOQCZXNHSCB2Y32OCV32ITSQOVV3DSD", "issuer": "Test1",
         "digits": 6, "type": "TOTP", "counter": 0, "algorithm": "SHA1"},
        {"label": "Test2", "secret": "XHRHVLZKO5YARFKEH65RFC7NVOOQCZXNHSCB2Y32OCV32ITSQOVV3DSD", "issuer": "Test2",
         "digits": 6, "type": "TOTP", "counter": 0, "algorithm": "SHA1"},
        {"label": "Test3", "secret": "XHRHVLZKO5YARFKEH65RFC7NVOOQCZXNHSCB2Y32OCV32ITSQOVV3DSD", "issuer": "Test3",
         "digits": 6, "type": "TOTP", "counter": 0, "algorithm": "SHA1"}
    ]

def test_freeotp_converter():
    assert freeotp_converter(freeotp_fetch_json("example_databases/freeotp.xml")) == [
        {"label": "TOTP512", "secret": "3ZUNXX2SU6RZP4QFMS32YAILJFS2I2T2UZXYSHSX7IIMPAQZAC753NG2", "issuer": "", "algorithm": "SHA512", "type": "TOTP", "digits": 6, "counter": 0}, 
        {"label": "TOTP256", "secret": "T2POIVOD6PZYO5YUNIULXIXB3SNXT4GYLSLO5FUNLIXOEMQHY2R6QAGB", "issuer": "", "algorithm": "SHA256", "type": "TOTP", "digits": 8, "counter": 0}, 
        {"label": "HOTP1", "secret": "QEXCCTBU5DXXSFZQIYQAZ7BZUV5TM4EAXMDGSN4TPMR5BSFTFVEXWPMX", "issuer": "", "algorithm": "SHA1", "type": "HOTP", "digits": 8, "counter": 1}, 
        {"label": "Test3", "secret": "XHRHVLZKO5YARFKEH65RFC7NVOOQCZXNHSCB2Y32OCV32ITSQOVV3DSD", "issuer": "", "algorithm": "SHA256", "type": "TOTP", "digits": 8, "counter": 0}, 
        {"label": "TOTP1", "secret": "MC6SGRDYDMCZBEROCAESWB4KBNTGN76C3NPUNAZMIA5G2CNHRJBB7L6W", "issuer": "", "algorithm": "SHA1", "type": "TOTP", "digits": 8, "counter": 0}
        ]



import base64
import sys
import xml.etree.ElementTree as Et
import json


def fetch_json(filename):
    element_list = []
    tree = Et.parse(filename)
    for e in tree.findall("string"):
        element_list.append(json.loads(e.text.replace("&quot;", "\"")))
    return element_list


def object_to_cotp_json(d):
    final = []
    for element in d:
        if type(element) is dict:
            final.append(
                {
                    'label': element['label'],
                    'secret': byte_array_to_base32(element['secret']),
                    'issuer': element['issuerExt'],
                    'algorithm': element['algo'],
                    'type': element['type'],
                    'digits': int(element['digits']),
                    'counter': element['counter'],
                }
            )
    return final

def byte_array_to_base32(secret: list):
    b = bytearray()
    for n in secret:
        # convert signed byte to unsigned
        b.append(n & 255)

    return base64.b32encode(b).decode('utf-8')

def main():
    if len(sys.argv) != 3:
        print("Usage: python authy.py [INPUT_FILE] [OUTPUT_FILE]")
        return

    data = fetch_json(sys.argv[1])
    output_file = open(sys.argv[2], 'w')
    output_file.write(json.dumps(object_to_cotp_json(data)))
    output_file.close()


main()



import sqlite3
import sys
import json


def get_accounts(filename):
    conn = sqlite3.connect(filename)
    c = conn.cursor()
    accounts = []
    for row in c.execute("SELECT email,secret,issuer,type,counter FROM accounts"):
        if row[3] == 0:
            type_ = "TOTP"
        else:
            type_ = "HOTP"
        accounts.append(
            {
                'label': row[0],
                'secret': row[1],
                'issuer': row[2],
                'type': type_,
                'digits': 6,
                'counter': row[4],
                'algorithm': 'SHA1',
            }
        )
    c.close()
    return accounts


def main():
    if len(sys.argv) != 3:
        print("Usage: python gauth.py [INPUT_FILE] [OUTPUT_FILE]")
        return
    output_file = open(sys.argv[2], 'w')
    output_file.write(json.dumps(get_accounts(sys.argv[1])))
    output_file.close()


main()
